@@ -57,9 +57,9 @@ GRL_LOG_DOMAIN_STATIC(apple_trailers_log_domain);
 
 typedef struct {
   GrlSourceBrowseSpec *bs;
+  GCancellable *cancellable;
   xmlDocPtr xml_doc;
   xmlNodePtr xml_entries;
-  gboolean cancelled;
 } OperationData;
 
 enum {
@@ -70,7 +70,6 @@ enum {
 
 struct _GrlAppleTrailersSourcePriv {
   GrlNetWc *wc;
-  GCancellable *cancellable;
   gboolean hd;
   gboolean large_poster;
 };
@@ -159,10 +158,15 @@ grl_apple_trailers_source_new (gboolean high_definition,
                                gboolean xlarge)
 {
   GrlAppleTrailersSource *source;
+  GIcon *icon;
+  GFile *file;
 
   GRL_DEBUG ("grl_apple_trailers_source_new%s%s",
              high_definition ? " (HD)" : "",
              xlarge ? " (X-large poster)" : "");
+  file = g_file_new_for_uri ("resource:///org/gnome/grilo/plugins/apple-trailers/channel-trailers.svg");
+  icon = g_file_icon_new (file);
+  g_object_unref (file);
   source = g_object_new (GRL_APPLE_TRAILERS_SOURCE_TYPE,
                          "source-id", SOURCE_ID,
                          "source-name", SOURCE_NAME,
@@ -170,6 +174,7 @@ grl_apple_trailers_source_new (gboolean high_definition,
                          "supported-media", GRL_MEDIA_TYPE_VIDEO,
                          "high-definition", high_definition,
 			 "large-poster", xlarge,
+                         "source-icon", icon,
                          NULL);
 
   return source;
@@ -187,10 +192,6 @@ grl_apple_trailers_source_finalize (GObject *object)
   if (self->priv->wc)
     g_object_unref (self->priv->wc);
 
-  if (self->priv->cancellable
-      && G_IS_CANCELLABLE (self->priv->cancellable))
-    g_object_unref (self->priv->cancellable);
-
   G_OBJECT_CLASS (grl_apple_trailers_source_parent_class)->finalize (object);
 }
 
@@ -306,7 +307,7 @@ runtime_to_seconds (const gchar *runtime)
   seconds = 0;
   items = g_strsplit (runtime, ":", -1);
   if (items && items[0] && items[1])
-    seconds = 3600 * atoi (items[0]) + 60 * atoi (items[1]);
+    seconds = 60 * atoi (items[0]) + atoi (items[1]);
   g_strfreev (items);
 
   return seconds;
@@ -403,7 +404,7 @@ send_movie_info (OperationData *op_data)
   GrlMedia *media;
   gboolean last = FALSE;
 
-  if (op_data->cancelled) {
+  if (g_cancellable_is_cancelled (op_data->cancellable)) {
     op_data->bs->callback (op_data->bs->source,
                            op_data->bs->operation_id,
                            NULL,
@@ -434,6 +435,7 @@ send_movie_info (OperationData *op_data)
 
   if (last) {
     xmlFreeDoc (op_data->xml_doc);
+    g_object_unref (op_data->cancellable);
     g_slice_free (OperationData, op_data);
   }
 
@@ -447,7 +449,7 @@ xml_parse_result (const gchar *str, OperationData *op_data)
   xmlNodePtr node;
   guint skip = grl_operation_options_get_skip (op_data->bs->options);
 
-  if (op_data->cancelled ||
+  if (g_cancellable_is_cancelled (op_data->cancellable) ||
       grl_operation_options_get_count (op_data->bs->options) == 0) {
     goto finalize;
   }
@@ -503,6 +505,7 @@ xml_parse_result (const gchar *str, OperationData *op_data)
     g_error_free (error);
   }
 
+  g_object_unref (op_data->cancellable);
   g_slice_free (OperationData, op_data);
 }
 
@@ -533,6 +536,7 @@ read_done_cb (GObject *source_object,
                            error);
     g_error_free (wc_error);
     g_error_free (error);
+    g_object_unref (op_data->cancellable);
     g_slice_free (OperationData, op_data);
 
     return;
@@ -544,19 +548,17 @@ read_done_cb (GObject *source_object,
 static void
 read_url_async (GrlAppleTrailersSource *source,
                 const gchar *url,
-                gpointer user_data)
+                OperationData *op_data)
 {
   if (!source->priv->wc)
     source->priv->wc = grl_net_wc_new ();
 
-  source->priv->cancellable = g_cancellable_new ();
-
   GRL_DEBUG ("Opening '%s'", url);
   grl_net_wc_request_async (source->priv->wc,
                         url,
-                        source->priv->cancellable,
+                        op_data->cancellable,
                         read_done_cb,
-                        user_data);
+                        op_data);
 }
 
 /* ================== API Implementation ================ */
@@ -594,6 +596,7 @@ grl_apple_trailers_source_browse (GrlSource *source,
 
   op_data = g_slice_new0 (OperationData);
   op_data->bs = bs;
+  op_data->cancellable = g_cancellable_new();
   grl_operation_set_data (bs->operation_id, op_data);
 
   if (at_source->priv->hd) {
@@ -607,21 +610,11 @@ static void
 grl_apple_trailers_source_cancel (GrlSource *source, guint operation_id)
 {
   OperationData *op_data;
-  GrlAppleTrailersSourcePriv *priv;
 
   GRL_DEBUG ("grl_apple_trailers_source_cancel");
 
-  priv = GRL_APPLE_TRAILERS_SOURCE_GET_PRIVATE (source);
-  if (priv->cancellable && G_IS_CANCELLABLE (priv->cancellable))
-    g_cancellable_cancel (priv->cancellable);
-  priv->cancellable = NULL;
-
-  if (priv->wc)
-    grl_net_wc_flush_delayed_requests (priv->wc);
-
   op_data = (OperationData *) grl_operation_get_data (operation_id);
-
   if (op_data) {
-    op_data->cancelled = TRUE;
+    g_cancellable_cancel (op_data->cancellable);
   }
 }
