@@ -149,11 +149,18 @@ GRL_PLUGIN_REGISTER (grl_bliptv_plugin_init,
 static GrlBliptvSource *
 grl_bliptv_source_new (void)
 {
+  GIcon *icon;
+  GFile *file;
+
+  file = g_file_new_for_uri ("resource:///org/gnome/grilo/plugins/bliptv/channel-bliptv.svg");
+  icon = g_file_icon_new (file);
+  g_object_unref (file);
   return g_object_new (GRL_TYPE_BLIPTV_SOURCE,
                        "source-id", SOURCE_ID,
                        "source-name", SOURCE_NAME,
                        "source-desc", SOURCE_DESC,
                        "supported-media", GRL_MEDIA_TYPE_VIDEO,
+                       "source-icon", icon,
                        NULL);
 }
 
@@ -431,8 +438,6 @@ call_raw_async_cb (GObject *     source_object,
                   NULL);
   }
 
-  bliptv_operation_free (op);
-
  finalize_free:
   if (xpath)
     xmlXPathFreeContext (xpath);
@@ -485,7 +490,7 @@ grl_bliptv_source_browse (GrlSource *source,
   op->callback     = bs->callback;
   op->user_data    = bs->user_data;
 
-  grl_operation_set_data (bs->operation_id, op);
+  grl_operation_set_data_full (bs->operation_id, op, (GDestroyNotify) bliptv_operation_free);
 
   url = g_strdup_printf (op->url, page_number);
 
@@ -527,7 +532,7 @@ grl_bliptv_source_search (GrlSource *source,
   op->user_data    = ss->user_data;
 
 
-  grl_operation_set_data (ss->operation_id, op);
+  grl_operation_set_data_full (ss->operation_id, op, (GDestroyNotify) bliptv_operation_free);
 
   url = g_strdup_printf (op->url, page_number);
 
@@ -547,11 +552,12 @@ grl_bliptv_source_cancel (GrlSource *source, guint operation_id)
 {
   BliptvOperation *op = grl_operation_get_data (operation_id);
 
-  GRL_WARNING ("Cancelling id=%u", operation_id);
+  GRL_DEBUG ("Cancelling id=%u", operation_id);
 
   if (!op)
     {
       GRL_WARNING ("\tNo such operation id=%u", operation_id);
+      return;
     }
 
   if (op->cancellable) {
