@@ -26,6 +26,7 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <glib/gi18n-lib.h>
 #include <grilo.h>
 #include <net/grl-net.h>
 #include <libxml/xpath.h>
@@ -151,7 +152,7 @@ GRL_LOG_DOMAIN_STATIC(podcasts_log_domain);
 
 #define SOURCE_ID   "grl-podcasts"
 #define SOURCE_NAME "Podcasts"
-#define SOURCE_DESC "A source for browsing podcasts"
+#define SOURCE_DESC _("A source for browsing podcasts")
 
 enum {
   PODCAST_ID = 0,
@@ -276,11 +277,19 @@ grl_podcasts_plugin_init (GrlRegistry *registry,
 
   GRL_DEBUG ("podcasts_plugin_init");
 
+  /* Initialize i18n */
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
   GrlPodcastsSource *source = grl_podcasts_source_new ();
+  g_object_add_weak_pointer (G_OBJECT (source), (gpointer *) &source);
   grl_registry_register_source (registry,
                                 plugin,
                                 GRL_SOURCE (source),
                                 NULL);
+  if (source == NULL)
+    return TRUE;
+  g_object_remove_weak_pointer (G_OBJECT (source), (gpointer *) &source);
 
   source->priv->cache_time = DEFAULT_CACHE_TIME;
   if (!configs || !configs->data) {
@@ -740,8 +749,9 @@ produce_podcast_contents_from_db (OperationSpec *os)
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to retrieve podcast streams: %s", sqlite3_errmsg (db));
     error = g_error_new (GRL_CORE_ERROR,
-			 os->error_code,
-			 "Failed to retrieve podcast streams");
+                         os->error_code,
+                         _("Failed to get podcast streams: %s"),
+                         sqlite3_errmsg (db));
     os->callback (os->source, os->operation_id, NULL, 0, os->user_data, error);
     g_error_free (error);
     return;
@@ -759,8 +769,9 @@ produce_podcast_contents_from_db (OperationSpec *os)
   if (r != SQLITE_DONE) {
     GRL_WARNING ("Failed to retrive podcast streams: %s", sqlite3_errmsg (db));
     error = g_error_new (GRL_CORE_ERROR,
-			 os->error_code,
-			 "Failed to retrieve podcast streams");
+                         os->error_code,
+                         _("Failed to get podcast streams: %s"),
+                         sqlite3_errmsg (db));
     os->callback (os->source, os->operation_id, NULL, 0, os->user_data, error);
     g_error_free (error);
     sqlite3_finalize (sql_stmt);
@@ -802,8 +813,9 @@ remove_podcast_streams (sqlite3 *db, const gchar *podcast_id, GError **error)
   if (r) {
     GRL_WARNING ("Failed to remove podcast streams cache: %s", sql_error);
     *error = g_error_new (GRL_CORE_ERROR,
-			  GRL_CORE_ERROR_REMOVE_FAILED,
-			  "Failed to remove podcast streams");
+                          GRL_CORE_ERROR_REMOVE_FAILED,
+                          _("Failed to remove: %s"),
+                          sql_error);
     sqlite3_free (error);
   }
 }
@@ -831,10 +843,11 @@ remove_podcast (GrlPodcastsSource *podcasts_source,
 
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to remove podcast '%s': %s", podcast_id, sql_error);
-    g_set_error_literal (error,
-                         GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_REMOVE_FAILED,
-                         "Failed to remove podcast");
+    g_set_error (error,
+                 GRL_CORE_ERROR,
+                 GRL_CORE_ERROR_REMOVE_FAILED,
+                 _("Failed to remove: %s"),
+                 sql_error);
     sqlite3_free (sql_error);
   } else if (podcasts_source->priv->notify_changes) {
     grl_source_notify_change (GRL_SOURCE (podcasts_source),
@@ -862,10 +875,11 @@ remove_stream (GrlPodcastsSource *podcasts_source,
 
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to remove podcast stream '%s': %s", url, sql_error);
-    g_set_error_literal (error,
-                         GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_REMOVE_FAILED,
-                         "Failed to remove podcast stream");
+    g_set_error (error,
+                 GRL_CORE_ERROR,
+                 GRL_CORE_ERROR_REMOVE_FAILED,
+                 _("Failed to remove: %s"),
+                 sql_error);
     sqlite3_free (sql_error);
   } else if (podcasts_source->priv->notify_changes) {
     grl_source_notify_change (GRL_SOURCE (podcasts_source),
@@ -905,7 +919,8 @@ store_podcast (GrlPodcastsSource *podcasts_source,
     g_set_error (error,
                  GRL_CORE_ERROR,
                  GRL_CORE_ERROR_STORE_FAILED,
-                 "Failed to store podcast '%s'", title);
+                 _("Failed to store: %s"),
+                 sqlite3_errmsg (podcasts_source->priv->db));
     return;
   }
 
@@ -937,7 +952,8 @@ store_podcast (GrlPodcastsSource *podcasts_source,
     g_set_error (error,
                  GRL_CORE_ERROR,
                  GRL_CORE_ERROR_STORE_FAILED,
-                 "Failed to store podcast '%s'", title);
+                 _("Failed to store: %s"),
+                 sqlite3_errmsg (podcasts_source->priv->db));
     sqlite3_finalize (sql_stmt);
     return;
   }
@@ -1218,18 +1234,18 @@ parse_feed (OperationSpec *os, const gchar *str, GError **error)
 
   doc = xmlParseDoc ((xmlChar *) str);
   if (!doc) {
-    *error = g_error_new (GRL_CORE_ERROR,
-			  os->error_code,
-			  "Failed to parse podcast contents");
+    *error = g_error_new_literal (GRL_CORE_ERROR,
+                                  os->error_code,
+                                  _("Failed to parse content"));
     goto free_resources;
   }
 
   /* Get feed stream list */
   xpathCtx = xmlXPathNewContext (doc);
   if (!xpathCtx) {
-    *error = g_error_new (GRL_CORE_ERROR,
-			  os->error_code,
-			  "Failed to parse podcast contents");
+    *error = g_error_new_literal (GRL_CORE_ERROR,
+                                  os->error_code,
+                                  _("Failed to parse content"));
     goto free_resources;
   }
 
@@ -1237,9 +1253,9 @@ parse_feed (OperationSpec *os, const gchar *str, GError **error)
   xpathObj = xmlXPathEvalExpression ((xmlChar *) "/rss/channel",
 				     xpathCtx);
   if(xpathObj == NULL) {
-    *error = g_error_new (GRL_CORE_ERROR,
-			  os->error_code,
-			  "Failed to parse podcast contents");
+    *error = g_error_new_literal (GRL_CORE_ERROR,
+                                  os->error_code,
+                                  _("Failed to parse content"));
     goto free_resources;
   }
 
@@ -1270,9 +1286,9 @@ parse_feed (OperationSpec *os, const gchar *str, GError **error)
   xpathObj = xmlXPathEvalExpression ((xmlChar *) "/rss/channel/item",
 				     xpathCtx);
   if(xpathObj == NULL) {
-    *error = g_error_new (GRL_CORE_ERROR,
-			  os->error_code,
-			  "Failed to parse podcast contents");
+    *error = g_error_new_literal (GRL_CORE_ERROR,
+                                  os->error_code,
+                                  _("Failed to parse podcast contents"));
     goto free_resources;
   }
 
@@ -1339,9 +1355,9 @@ read_feed_cb (gchar *xmldata, gpointer user_data)
   OperationSpec *os = (OperationSpec *) user_data;
 
   if (!xmldata) {
-    error = g_error_new (GRL_CORE_ERROR,
-			 GRL_CORE_ERROR_BROWSE_FAILED,
-			 "Failed to read data from podcast");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_BROWSE_FAILED,
+                                 _("Empty response"));
   } else {
     parse_feed (os, xmldata, &error);
   }
@@ -1428,9 +1444,9 @@ produce_podcast_contents (OperationSpec *os)
     }
     sqlite3_finalize (sql_stmt);
   } else {
-    error = g_error_new (GRL_CORE_ERROR,
-			 os->error_code,
-			 "Failed to retrieve podcast information");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 os->error_code,
+                                 _("Failed to get podcast information"));
     os->callback (os->source, os->operation_id, NULL, 0, os->user_data, error);
     g_error_free (error);
     g_slice_free (OperationSpec, os);
@@ -1469,8 +1485,9 @@ produce_podcasts (OperationSpec *os)
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to retrieve podcasts: %s", sqlite3_errmsg (db));
     error = g_error_new (GRL_CORE_ERROR,
-			 os->error_code,
-			 "Failed to retrieve podcasts list");
+                         os->error_code,
+                         _("Failed to get podcasts list: %s"),
+                         sqlite3_errmsg (db));
     os->callback (os->source, os->operation_id, NULL, 0, os->user_data, error);
     g_error_free (error);
     goto free_resources;
@@ -1488,8 +1505,9 @@ produce_podcasts (OperationSpec *os)
   if (r != SQLITE_DONE) {
     GRL_WARNING ("Failed to retrieve podcasts: %s", sqlite3_errmsg (db));
     error = g_error_new (GRL_CORE_ERROR,
-			 os->error_code,
-			 "Failed to retrieve podcasts list");
+                         os->error_code,
+                         _("Failed to get podcasts list: %s"),
+                         sqlite3_errmsg (db));
     os->callback (os->source, os->operation_id, NULL, 0, os->user_data, error);
     g_error_free (error);
     goto free_resources;
@@ -1540,9 +1558,9 @@ stream_resolve (GrlSourceResolveSpec *rs)
 
   if (r != SQLITE_OK) {
     GRL_WARNING ("Failed to get podcast stream: %s", sqlite3_errmsg (db));
-    error = g_error_new (GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_RESOLVE_FAILED,
-                         "Failed to get podcast stream metadata");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_RESOLVE_FAILED,
+                                 _("Failed to get podcast stream metadata"));
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
     g_error_free (error);
     return;
@@ -1555,9 +1573,9 @@ stream_resolve (GrlSourceResolveSpec *rs)
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, NULL);
   } else {
     GRL_WARNING ("Failed to get podcast stream: %s", sqlite3_errmsg (db));
-    error = g_error_new (GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_RESOLVE_FAILED,
-                         "Failed to get podcast stream metadata");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_RESOLVE_FAILED,
+                                 _("Failed to get podcast stream metadata"));
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
     g_error_free (error);
   }
@@ -1593,9 +1611,9 @@ podcast_resolve (GrlSourceResolveSpec *rs)
     sqlite3_finalize (sql_stmt);
   } else {
     GRL_WARNING ("Failed to get podcast: %s", sqlite3_errmsg (db));
-    error = g_error_new (GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_RESOLVE_FAILED,
-                         "Failed to get podcast metadata");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_RESOLVE_FAILED,
+                                 _("Failed to get podcast metadata"));
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
     g_error_free (error);
   }
@@ -1637,9 +1655,9 @@ grl_podcasts_source_browse (GrlSource *source,
   podcasts_source = GRL_PODCASTS_SOURCE (source);
   if (!podcasts_source->priv->db) {
     GRL_WARNING ("Can't execute operation: no database connection.");
-    error = g_error_new (GRL_CORE_ERROR,
-			 GRL_CORE_ERROR_BROWSE_FAILED,
-			 "No database connection");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_BROWSE_FAILED,
+                                 _("No database connection"));
     bs->callback (bs->source, bs->operation_id, NULL, 0, bs->user_data, error);
     g_error_free (error);
     return;
@@ -1681,9 +1699,9 @@ grl_podcasts_source_search (GrlSource *source,
   podcasts_source = GRL_PODCASTS_SOURCE (source);
   if (!podcasts_source->priv->db) {
     GRL_WARNING ("Can't execute operation: no database connection.");
-    error = g_error_new (GRL_CORE_ERROR,
-			 GRL_CORE_ERROR_QUERY_FAILED,
-			 "No database connection");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_QUERY_FAILED,
+                                 _("No database connection"));
     ss->callback (ss->source, ss->operation_id, NULL, 0, ss->user_data, error);
     g_error_free (error);
     return;
@@ -1715,9 +1733,9 @@ grl_podcasts_source_query (GrlSource *source, GrlSourceQuerySpec *qs)
   podcasts_source = GRL_PODCASTS_SOURCE (source);
   if (!podcasts_source->priv->db) {
     GRL_WARNING ("Can't execute operation: no database connection.");
-    error = g_error_new (GRL_CORE_ERROR,
-			 GRL_CORE_ERROR_QUERY_FAILED,
-			 "No database connection");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_QUERY_FAILED,
+                                 _("No database connection"));
     qs->callback (qs->source, qs->operation_id, NULL, 0, qs->user_data, error);
     g_error_free (error);
     return;
@@ -1750,9 +1768,9 @@ grl_podcasts_source_resolve (GrlSource *source,
   podcasts_source = GRL_PODCASTS_SOURCE (source);
   if (!podcasts_source->priv->db) {
     GRL_WARNING ("Can't execute operation: no database connection.");
-    error = g_error_new (GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_RESOLVE_FAILED,
-                         "No database connection");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_RESOLVE_FAILED,
+                                 _("No database connection"));
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
     g_error_free (error);
     return;
@@ -1777,13 +1795,14 @@ grl_podcasts_source_store (GrlSource *source, GrlSourceStoreSpec *ss)
   keylist = grl_data_get_keys (GRL_DATA (ss->media));
 
   if (GRL_IS_MEDIA_BOX (ss->media)) {
-    error = g_error_new (GRL_CORE_ERROR,
-                         GRL_CORE_ERROR_STORE_FAILED,
-                         "Cannot create containers. Only feeds are accepted.");
+    error = g_error_new_literal (GRL_CORE_ERROR,
+                                 GRL_CORE_ERROR_STORE_FAILED,
+                                 _("Cannot create containers. Only feeds are accepted"));
   } else if (!grl_data_has_key (GRL_DATA (ss->media), GRL_METADATA_KEY_URL)) {
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_STORE_FAILED,
-                         "Cannot store podcast. URL required");
+                         _("Failed to store: %s"),
+                         _("URL required"));
   } else {
     store_podcast (GRL_PODCASTS_SOURCE (ss->source),
                    &keylist,
