@@ -206,7 +206,6 @@ gboolean grl_raitv_plugin_init (GrlRegistry *registry,
                                 GList *configs);
 
 static const GList *grl_raitv_source_supported_keys (GrlSource *source);
-static const GList *grl_raitv_source_slow_keys (GrlSource *source);
 
 static void grl_raitv_source_browse (GrlSource *source,
                                      GrlSourceBrowseSpec *bs);
@@ -259,11 +258,18 @@ GRL_PLUGIN_REGISTER (grl_raitv_plugin_init,
 static GrlRaitvSource *
 grl_raitv_source_new (void)
 {
+  GIcon *icon;
+  GFile *file;
+
+  file = g_file_new_for_uri ("resource:///org/gnome/grilo/plugins/raitv/channel-rai.svg");
+  icon = g_file_icon_new (file);
+  g_object_unref (file);
   return g_object_new (GRL_TYPE_RAITV_SOURCE,
                        "source-id", SOURCE_ID,
                        "source-name", SOURCE_NAME,
                        "source-desc", SOURCE_DESC,
                        "supported-media", GRL_MEDIA_TYPE_VIDEO,
+                       "source-icon", icon,
                        NULL);
 }
 
@@ -309,7 +315,6 @@ grl_raitv_source_class_init (GrlRaitvSourceClass *klass)
   object_class->finalize = grl_raitv_source_finalize;
 
   source_class->supported_keys = grl_raitv_source_supported_keys;
-  source_class->slow_keys = grl_raitv_source_slow_keys;
 
   source_class->cancel = grl_raitv_source_cancel;
   source_class->browse = grl_raitv_source_browse;
@@ -564,7 +569,6 @@ proxy_call_search_grlnet_async_cb (GObject *source_object,
                   0,
                   op->user_data,
                   NULL);
-    raitv_operation_free (op);
   }
   else {
     //Continue the search
@@ -774,7 +778,6 @@ proxy_call_browse_grlnet_async_cb (GObject *source_object,
                   0,
                   op->user_data,
                   NULL);
-    raitv_operation_free (op);
   }
   else {
     //Continue the search
@@ -953,17 +956,17 @@ proxy_call_resolve_grlnet_async_cb (GObject *source_object,
 
 
 
-static gint
+static guint
 get_theme_index_from_id (const gchar *category_id)
 {
-  gint i;
+  guint i;
 
   for (i=0; i<root_dir[ROOT_DIR_POPULARS_INDEX].count; i++) {
     if (g_strrstr (category_id, themes_dir[i].id)) {
       return i;
     }
   }
-  return -1;
+  g_assert_not_reached ();
 }
 
 static gboolean
@@ -1065,7 +1068,7 @@ produce_from_directory (CategoryInfo *dir, gint dir_size, RaitvOperation *os,
 
     do {
       GrlMedia *content =
-        produce_container_from_directory (NULL, dir, index,type );
+        produce_container_from_directory (NULL, dir, index, type);
 
       remaining--;
       index++;
@@ -1084,7 +1087,7 @@ produce_from_directory (CategoryInfo *dir, gint dir_size, RaitvOperation *os,
 static void
 produce_from_popular_theme (RaitvOperation *op)
 {
-  gint category_index;
+  guint category_index;
   gchar	*start = NULL;
   gchar	*url = NULL;
 
@@ -1112,7 +1115,7 @@ produce_from_popular_theme (RaitvOperation *op)
 static void
 produce_from_recent_theme (RaitvOperation *op)
 {
-  gint category_index;
+  guint category_index;
   gchar	*start = NULL;
   gchar	*url = NULL;
 
@@ -1154,18 +1157,6 @@ grl_raitv_source_supported_keys (GrlSource *source)
   return keys;
 }
 
-static const GList *
-grl_raitv_source_slow_keys (GrlSource *source)
-{
-  static GList *keys = NULL;
-  if (!keys) {
-    keys = grl_metadata_key_list_new (GRL_METADATA_KEY_URL,
-                                      NULL);
-  }
-  return keys;
-}
-
-
 static void
 grl_raitv_source_browse (GrlSource *source,
                          GrlSourceBrowseSpec *bs)
@@ -1187,7 +1178,7 @@ grl_raitv_source_browse (GrlSource *source,
   op->count	   = op->length;
   op->offset       = 0;
 
-  grl_operation_set_data (bs->operation_id, op);
+  grl_operation_set_data_full (bs->operation_id, op, (GDestroyNotify) raitv_operation_free);
 
   RaitvMediaType type = classify_media_id (container_id);
   switch (type)
@@ -1234,7 +1225,7 @@ grl_raitv_source_search (GrlSource *source,
   op->offset       = 0;
   op->text	   = ss->text;
 
-  grl_operation_set_data (ss->operation_id, op);
+  grl_operation_set_data_full (ss->operation_id, op, (GDestroyNotify) raitv_operation_free);
 
   g_raitv_videos_search(op);
 }
@@ -1306,7 +1297,7 @@ grl_raitv_source_resolve (GrlSource *source,
     op->user_data    = rs->user_data;
     op->media	      = rs->media;
 
-    grl_operation_set_data (rs->operation_id, op);
+    grl_operation_set_data_full (rs->operation_id, op, (GDestroyNotify) raitv_operation_free);
 
     urltarget = g_strdup_printf ("http://www.rai.tv/dl/RaiTV/programmi/media/%s.html",
                                  grl_media_get_id(rs->media));
@@ -1338,7 +1329,7 @@ grl_raitv_source_resolve (GrlSource *source,
   op->user_data    = rs->user_data;
   op->media	   = rs->media;
 
-  grl_operation_set_data (rs->operation_id, op);
+  grl_operation_set_data_full (rs->operation_id, op, (GDestroyNotify) raitv_operation_free);
 
   urltarget = g_strdup_printf("%s/%s.html","http://www.rai.tv/dl/RaiTV/programmi/media",grl_media_get_id(rs->media));
 
@@ -1359,11 +1350,12 @@ grl_raitv_source_cancel (GrlSource *source, guint operation_id)
 {
   RaitvOperation *op = grl_operation_get_data (operation_id);
 
-  GRL_WARNING ("Cancelling id=%u", operation_id);
+  GRL_DEBUG ("Cancelling id=%u", operation_id);
 
   if (!op)
     {
       GRL_WARNING ("\tNo such operation id=%u", operation_id);
+      return;
     }
 
   if (op->cancellable) {
