@@ -57,12 +57,19 @@ GRL_LOG_DOMAIN_STATIC(tracker_source_result_log_domain);
 
 /* ------- Definitions ------- */
 
-#define TRACKER_QUERY_REQUEST                   \
+#define TRACKER_QUERY_LIMIT                     \
+  "OFFSET %u "                                  \
+  "LIMIT %u"
+
+#define TRACKER_QUERY_PARTIAL_REQUEST           \
   "SELECT rdf:type(?urn) %s "                   \
   "WHERE { %s . %s } "                          \
   "ORDER BY DESC(nfo:fileLastModified(?urn)) "  \
-  "OFFSET %u "                                  \
-  "LIMIT %u"
+  TRACKER_QUERY_LIMIT
+
+#define TRACKER_QUERY_FULL_REQUEST              \
+  "%s "                                         \
+  TRACKER_QUERY_LIMIT
 
 #define TRACKER_SEARCH_REQUEST                  \
   "SELECT rdf:type(?urn) %s "                   \
@@ -182,13 +189,23 @@ fill_grilo_media_from_sparql (GrlTrackerSource    *source,
     const gchar *str_val;
   } val;
 
-  if (assoc == NULL)
-    return;
+  GrlKeyID grl_key;
+
+  if (assoc == NULL) {
+    /* Maybe the user is setting the key */
+    GrlRegistry *registry = grl_registry_get_default ();
+    grl_key = grl_registry_lookup_metadata_key (registry, sparql_key);
+    if (grl_key == GRL_METADATA_KEY_INVALID) {
+      return;
+    }
+  } else {
+    grl_key = assoc->grl_key;
+  }
 
   GRL_ODEBUG ("\tSetting media prop (col=%i/var=%s/prop=%s) %s",
               column,
               sparql_key,
-              GRL_METADATA_KEY_GET_NAME (assoc->grl_key),
+              GRL_METADATA_KEY_GET_NAME (grl_key),
               tracker_sparql_cursor_get_string (cursor, column, NULL));
 
   if (tracker_sparql_cursor_is_bound (cursor, column) == FALSE) {
@@ -196,41 +213,39 @@ fill_grilo_media_from_sparql (GrlTrackerSource    *source,
     return;
   }
 
-  if (grl_data_has_key (GRL_DATA (media), assoc->grl_key)) {
+  if (grl_data_has_key (GRL_DATA (media), grl_key)) {
     GRL_ODEBUG ("\t\tDropping, already here");
     return;
   }
 
-  if (assoc->set_value) {
+  if (assoc && assoc->set_value) {
     assoc->set_value (cursor, column, media, assoc->grl_key);
   } else {
-    switch (GRL_METADATA_KEY_GET_TYPE (assoc->grl_key)) {
-      case G_TYPE_STRING:
-        /* Cache the source associated to this result. */
-        if (assoc->grl_key == GRL_METADATA_KEY_ID) {
-          grl_tracker_source_cache_add_item (grl_tracker_item_cache,
-                                             tracker_sparql_cursor_get_integer (cursor,
-                                                                                column),
-                                             source);
-        }
-        val.str_val = tracker_sparql_cursor_get_string (cursor, column, NULL);
-        if (val.str_val != NULL)
-          grl_data_set_string (GRL_DATA (media), assoc->grl_key, val.str_val);
-        break;
-
-      case G_TYPE_INT:
-        val.int_val = tracker_sparql_cursor_get_integer (cursor, column);
-        grl_data_set_int (GRL_DATA (media), assoc->grl_key, val.int_val);
-        break;
-
-      case G_TYPE_FLOAT:
-        val.double_val = tracker_sparql_cursor_get_double (cursor, column);
-        grl_data_set_float (GRL_DATA (media), assoc->grl_key, (gfloat) val.double_val);
-        break;
-
-      default:
-        GRL_ODEBUG ("\t\tUnexpected data type");
-        break;
+    GType grl_type = GRL_METADATA_KEY_GET_TYPE (grl_key);
+    if (grl_type == G_TYPE_STRING) {
+      /* Cache the source associated to this result. */
+      if (grl_key == GRL_METADATA_KEY_ID) {
+        grl_tracker_source_cache_add_item (grl_tracker_item_cache,
+                                           tracker_sparql_cursor_get_integer (cursor,
+                                                                              column),
+                                           source);
+      }
+      val.str_val = tracker_sparql_cursor_get_string (cursor, column, NULL);
+      if (val.str_val != NULL)
+        grl_data_set_string (GRL_DATA (media), grl_key, val.str_val);
+    } else if (grl_type == G_TYPE_INT) {
+      val.int_val = tracker_sparql_cursor_get_integer (cursor, column);
+      grl_data_set_int (GRL_DATA (media), grl_key, val.int_val);
+    } else if (grl_type == G_TYPE_FLOAT) {
+      val.double_val = tracker_sparql_cursor_get_double (cursor, column);
+      grl_data_set_float (GRL_DATA (media), grl_key, (gfloat) val.double_val);
+    } else if (grl_type == G_TYPE_DATE_TIME) {
+      val.str_val = tracker_sparql_cursor_get_string (cursor, column, NULL);
+      GDateTime *date_time = grl_date_time_from_iso8601 (val.str_val);
+      grl_data_set_boxed (GRL_DATA (media), grl_key, date_time);
+      g_date_time_unref (date_time);
+    } else {
+      GRL_ODEBUG ("\t\tUnexpected data type");
     }
   }
 }
@@ -314,7 +329,7 @@ get_sparql_type_filter (GrlOperationOptions *options,
                                                                         \
         error = g_error_new (GRL_CORE_ERROR,                            \
                              GRL_CORE_ERROR_BROWSE_FAILED,              \
-                             "Failed to start query action : %s",       \
+                             _("Failed to query: %s"),                  \
                              tracker_error->message);                   \
                                                                         \
         spec->callback (spec->source,                      \
@@ -396,7 +411,7 @@ get_sparql_type_filter (GrlOperationOptions *options,
                                                                         \
       error = g_error_new (GRL_CORE_ERROR,                              \
                            GRL_CORE_ERROR_BROWSE_FAILED,                \
-                           "Failed to start query action : %s",         \
+                           _("Failed to query: %s"),                    \
                            tracker_error->message);                     \
                                                                         \
       spec->callback (spec->source, spec->operation_id, NULL, 0,           \
@@ -442,7 +457,7 @@ tracker_resolve_cb (GObject      *source_object,
 
     error = g_error_new (GRL_CORE_ERROR,
                         GRL_CORE_ERROR_BROWSE_FAILED,
-                        "Failed to start resolve action : %s",
+                         _("Failed to resolve: %s"),
                          tracker_error->message);
 
     rs->callback (rs->source, rs->operation_id, rs->media, rs->user_data, error);
@@ -493,7 +508,7 @@ tracker_store_metadata_cb (GObject      *source_object,
 
     error = g_error_new (GRL_CORE_ERROR,
                          GRL_CORE_ERROR_STORE_METADATA_FAILED,
-                         "Failed to set metadata : %s",
+                         _("Failed to update metadata: %s"),
                          tracker_error->message);
 
     sms->callback (sms->source, sms->media, NULL, sms->user_data, error);
@@ -610,7 +625,7 @@ grl_tracker_source_query (GrlSource *source,
   if (!qs->query || qs->query[0] == '\0') {
     error = g_error_new_literal (GRL_CORE_ERROR,
                                  GRL_CORE_ERROR_QUERY_FAILED,
-                                 "Empty query");
+                                 _("Empty query"));
     goto send_error;
   }
 
@@ -618,7 +633,7 @@ grl_tracker_source_query (GrlSource *source,
   if (g_ascii_strncasecmp (qs->query, "select ", 7) != 0) {
     constraint = grl_tracker_source_get_device_constraint (priv);
     sparql_select = grl_tracker_source_get_select_string (qs->keys);
-    sparql_final = g_strdup_printf (TRACKER_QUERY_REQUEST,
+    sparql_final = g_strdup_printf (TRACKER_QUERY_PARTIAL_REQUEST,
                                     sparql_select,
                                     qs->query,
                                     constraint,
@@ -628,12 +643,16 @@ grl_tracker_source_query (GrlSource *source,
     g_free (qs->query);
     g_free (sparql_select);
     qs->query = sparql_final;
-    grl_tracker_source_query (source, qs);
-    return;
+  } else {
+    /* Append offset and limit */
+    sparql_final = g_strdup_printf (TRACKER_QUERY_FULL_REQUEST,
+                                    qs->query,
+                                    skip,
+                                    count);
+    g_free (qs->query);
+    qs->query = sparql_final;
   }
 
-  GRL_IDEBUG ("\tselect : '%s'", qs->query);
-
   os = grl_tracker_op_initiate_query (qs->operation_id,
                                       g_strdup (qs->query),
                                       (GAsyncReadyCallback) tracker_query_cb,
