@@ -33,6 +33,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
+#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 
 #include "grl-vimeo.h"
@@ -79,6 +80,7 @@ struct _GrlVimeoSourcePrivate {
 };
 
 static GrlVimeoSource *grl_vimeo_source_new (void);
+static void grl_vimeo_source_finalize (GObject *object);
 
 gboolean grl_vimeo_plugin_init (GrlRegistry *registry,
                                 GrlPlugin *plugin,
@@ -178,13 +180,21 @@ GRL_PLUGIN_REGISTER (grl_vimeo_plugin_init,
 static GrlVimeoSource *
 grl_vimeo_source_new (void)
 {
+  GIcon *icon;
+  GFile *file;
+
   GRL_DEBUG ("grl_vimeo_source_new");
 
+  file = g_file_new_for_uri ("resource:///org/gnome/grilo/plugins/vimeo/channel-vimeo.svg");
+  icon = g_file_icon_new (file);
+  g_object_unref (file);
+
   return g_object_new (GRL_VIMEO_SOURCE_TYPE,
                        "source-id", SOURCE_ID,
                        "source-name", SOURCE_NAME,
                        "source-desc", SOURCE_DESC,
                        "supported-media", GRL_MEDIA_TYPE_VIDEO,
+                       "source-icon", icon,
                        NULL);
 }
 
@@ -192,12 +202,15 @@ static void
 grl_vimeo_source_class_init (GrlVimeoSourceClass * klass)
 {
   GrlSourceClass *source_class = GRL_SOURCE_CLASS (klass);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   source_class->supported_keys = grl_vimeo_source_supported_keys;
   source_class->slow_keys = grl_vimeo_source_slow_keys;
   source_class->resolve = grl_vimeo_source_resolve;
   source_class->search = grl_vimeo_source_search;
 
+  object_class->finalize = grl_vimeo_source_finalize;
+
   g_type_class_add_private (klass, sizeof (GrlVimeoSourcePrivate));
 }
 
@@ -211,6 +224,16 @@ grl_vimeo_source_init (GrlVimeoSource *source)
 
 G_DEFINE_TYPE (GrlVimeoSource, grl_vimeo_source, GRL_TYPE_SOURCE);
 
+static void
+grl_vimeo_source_finalize (GObject *object)
+{
+  GrlVimeoSource *source = GRL_VIMEO_SOURCE (object);
+
+  g_clear_object (&source->priv->vimeo);
+
+  G_OBJECT_CLASS (grl_vimeo_source_parent_class)->finalize (object);
+}
+
 /* ======================= Utilities ==================== */
 
 static gint
