@@ -318,6 +318,8 @@ grl_youtube_source_new (const gchar *api_key, const gchar *client_id, const gcha
 {
   GrlYoutubeSource *source;
   GDataYouTubeService *service;
+  GIcon *icon;
+  GFile *file;
 
   GRL_DEBUG ("grl_youtube_source_new");
 
@@ -331,6 +333,10 @@ grl_youtube_source_new (const gchar *api_key, const gchar *client_id, const gcha
     return NULL;
   }
 
+  file = g_file_new_for_uri ("resource:///org/gnome/grilo/plugins/youtube/channel-youtube.svg");
+  icon = g_file_icon_new (file);
+  g_object_unref (file);
+
   /* Use auto-split mode because YouTube fails for queries
      that request more than YOUTUBE_MAX_CHUNK results */
   source = GRL_YOUTUBE_SOURCE (g_object_new (GRL_YOUTUBE_SOURCE_TYPE,
@@ -341,9 +347,11 @@ grl_youtube_source_new (const gchar *api_key, const gchar *client_id, const gcha
 					     YOUTUBE_MAX_CHUNK,
                                              "yt-service", service,
                                              "supported-media", GRL_MEDIA_TYPE_VIDEO,
+                                             "source-icon", icon,
 					     NULL));
 
   ytsrc = source;
+  g_object_add_weak_pointer (G_OBJECT (source), (gpointer *) &ytsrc);
 
   return source;
 }
@@ -480,6 +488,8 @@ get_wc (void)
 {
   if (ytsrc && !ytsrc->priv->wc)
     ytsrc->priv->wc = grl_net_wc_new ();
+  else if (!ytsrc)
+    return NULL;
 
   return ytsrc->priv->wc;
 }
@@ -518,6 +528,11 @@ read_url_async (const gchar *url,
                 gpointer user_data)
 {
   AsyncReadCb *arc;
+  GrlNetWc *wc;
+
+  wc = get_wc ();
+  if (!wc)
+    return;
 
   arc = g_slice_new0 (AsyncReadCb);
   arc->url = g_strdup (url);
@@ -525,7 +540,7 @@ read_url_async (const gchar *url,
   arc->user_data = user_data;
 
   GRL_DEBUG ("Opening async '%s'", url);
-  grl_net_wc_request_async (get_wc (),
+  grl_net_wc_request_async (wc,
                         url,
                         cancellable,
                         read_done_cb,
@@ -1063,9 +1078,6 @@ produce_from_directory (CategoryInfo *dir, gint dir_size, OperationSpec *os)
 
   GRL_DEBUG ("produce_from_directory");
 
-  /* YouTube's first index is 1, but the directories start at 0 */
-  os->skip--;
-
   if (os->skip >= dir_size) {
     /* No results */
     os->callback (os->source,
@@ -1144,7 +1156,9 @@ produce_from_feed (OperationSpec *os)
   grl_operation_set_data (os->operation_id, os->cancellable);
 
   service = GRL_YOUTUBE_SOURCE (os->source)->priv->service;
-  query = gdata_query_new_with_limits (NULL , os->skip, os->count);
+
+  /* Index in GData starts at 1 */
+  query = gdata_query_new_with_limits (NULL , os->skip + 1, os->count);
   os->category_info = &feeds_dir[feed_type];
 
 #ifdef HAVE_LIBGDATA_0_9
@@ -1203,7 +1217,9 @@ produce_from_category (OperationSpec *os)
   operation_spec_ref (os);
 
   service = GRL_YOUTUBE_SOURCE (os->source)->priv->service;
-  query = gdata_query_new_with_limits (NULL , os->skip, os->count);
+
+  /* Index in GData starts at 1 */
+  query = gdata_query_new_with_limits (NULL , os->skip + 1, os->count);
   os->category_info = &categories_dir[category_index];
   gdata_query_set_categories (query, category_term);
 
@@ -1392,7 +1408,7 @@ grl_youtube_source_search (GrlSource *source,
   os->cancellable = g_cancellable_new ();
   os->operation_id = ss->operation_id;
   os->keys = ss->keys;
-  os->skip = grl_operation_options_get_skip (ss->options) + 1;
+  os->skip = grl_operation_options_get_skip (ss->options);
   os->count = grl_operation_options_get_count (ss->options);
   os->callback = ss->callback;
   os->user_data = ss->user_data;
@@ -1403,7 +1419,8 @@ grl_youtube_source_search (GrlSource *source,
 
   grl_operation_set_data (ss->operation_id, os->cancellable);
 
-  query = gdata_query_new_with_limits (ss->text, os->skip, os->count);
+  /* Index in GData starts at 1 */
+  query = gdata_query_new_with_limits (ss->text, os->skip + 1, os->count);
 
 #ifdef HAVE_LIBGDATA_0_9
   gdata_youtube_service_query_videos_async (GDATA_YOUTUBE_SERVICE (GRL_YOUTUBE_SOURCE (source)->priv->service),
@@ -1435,7 +1452,11 @@ grl_youtube_source_browse (GrlSource *source,
   OperationSpec *os;
   const gchar *container_id;
 
-  GRL_DEBUG ("%s: %s", __FUNCTION__, grl_media_get_id (bs->container));
+  GRL_DEBUG ("%s: %s (%u, %d)",
+             __FUNCTION__,
+             grl_media_get_id (bs->container),
+             grl_operation_options_get_skip (bs->options),
+             grl_operation_options_get_count (bs->options));
 
   container_id = grl_media_get_id (bs->container);
 
@@ -1445,7 +1466,7 @@ grl_youtube_source_browse (GrlSource *source,
   os->container_id = container_id;
   os->keys = bs->keys;
   os->flags = grl_operation_options_get_flags (bs->options);
-  os->skip = grl_operation_options_get_skip (bs->options) + 1;
+  os->skip = grl_operation_options_get_skip (bs->options);
   os->count = grl_operation_options_get_count (bs->options);
   os->callback = bs->callback;
   os->user_data = bs->user_data;
